let s:old_titlestring=&titlestring
let s:old_tagrelative=&tagrelative
let s:old_tags=&tags

" exconfig#apply_project_type {{{
function exconfig#apply_project_type()
endfunction

" exconfig#reset {{{
function exconfig#reset()
    let &titlestring=s:old_titlestring
    let &tagrelative=s:old_tagrelative
    let &tags=s:old_tags
endfunction

" exconfig#apply {{{
function exconfig#apply()

    " ===================================
    " pre-check
    " ===================================

    let project_name = vimentry#get('project_name')
    if project_name == ''
        call ex#error("Can't find vimentry setting 'project_name'.")
        return
    endif

    " NOTE: we use the dir path of .vimentry instead of getcwd().  
    " getcwd
    let filename = expand('%')
    let cwd = ex#path#translate( fnamemodify( filename, ':p:h' ), 'unix' )

    let g:exvim_project_name = project_name
    let g:exvim_project_root = cwd
    let g:exvim_folder = './.exvim.'.project_name

    " set parent working directory
    silent exec 'cd ' . fnameescape(cwd)
    let g:exvim_project_name = project_name

    let s:old_titlestring=&titlestring
    set titlestring=%{g:exvim_project_name}:\ %t\ (%{expand(\"%:p:.:h\")}/)

    " create folder .exvim.xxx/ if not exists
    let path = g:exvim_folder
    if finddir(path) == ''
        silent call mkdir(path)
    endif

    " create folder .exvim.xxx/view/ if not exists, and set viewdir to it
    let path = g:exvim_folder.'/view'
    if finddir(path) == ''
        silent call mkdir(path)
    endif
    let &viewdir = g:exvim_folder . '/view'

    " ===================================
    " general settings
    " ===================================

    " apply project_type settings
    if !vimentry#check('project_type', '')
        " TODO:
        " let project_types = split( vimentry#get('project_type'), ',' )
    endif

    " set folder filter to g:exvim_root_folders and g:exvim_root_exclude_folders
    let folder_filter = copy(vimentry#get('folder_filter', []))
    let g:exvim_root_folders = []
    let g:exvim_root_exclude_folders = []

    if !empty(folder_filter)
        " we need search the root directory, and add folders that not excluded
        if vimentry#check('folder_filter_mode', 'exclude')
            " set include folders
            let filelist = split(globpath(cwd,'*'),'\n')
            for name in filelist
                if isdirectory(name)
                    let name = fnamemodify(name,':t')
                    if index( folder_filter, name ) == -1
                        silent call add ( g:exvim_root_folders, name )
                    endif
                endif
            endfor

            " set exclude folders
            let g:exvim_root_exclude_folders = copy(folder_filter)
        else
            " set include folders
            let g:exvim_root_folders = copy(folder_filter)

            " set exclude folders
            let filelist = split(globpath(cwd,'*'),'\n')
            for name in filelist
                if isdirectory(name)
                    let name = fnamemodify(name,':t')
                    if index( folder_filter, name ) == -1
                        silent call add ( g:exvim_root_exclude_folders, name )
                    endif
                endif
            endfor
        endif
    else
        let filelist = split(globpath(cwd,'*'),'\n')
        for name in filelist
            if isdirectory(name)
                let name = fnamemodify(name,':t')
                silent call add ( g:exvim_root_folders, name )
            endif
        endfor
    endif

    " set tag file path
    if vimentry#check('enable_tags', 'true')
        let s:old_tagrelative=&tagrelative
        let &tagrelative=0 " set notagrelative

        let s:old_tags=&tags
        let &tags=fnameescape(g:exvim_folder.'/tags')

        call exconfig#gen_sh_update_files(g:exvim_folder)
        call exconfig#gen_sh_update_ctags(g:exvim_folder)

        " symbols
        if vimentry#check('enable_symbols', 'true')
            call exsymbol#set_file(g:exvim_folder.'/symbols')
            call ex#set_symbol_path(g:exvim_folder.'/symbols')
            call exconfig#gen_sh_update_symbols(g:exvim_folder)
        endif

        " create .exvim.xxx/hierarchies/
        if vimentry#check('enable_inherits', 'true')
            " TODO:
            " let inherit_directory_path = g:exES_CWD.'/'.g:exES_vimfiles_dirname.'/.hierarchies'
            " if finddir(inherit_directory_path) == ''
            "   silent call mkdir(inherit_directory_path)
            " endif
        endif
    endif

    " set gsearch
    if vimentry#check('enable_gsearch', 'true')
        let gsearch_engine = vimentry#get('gsearch_engine')
        if gsearch_engine == 'idutils'
            call exgsearch#set_id_file(g:exvim_folder.'/ID')
            call exconfig#gen_sh_update_idutils(g:exvim_folder)
        endif
    endif

    " set cscope file path
    if vimentry#check('enable_cscope', 'true')
        " TODO: silent call g:exCS_ConnectCscopeFile()
    endif

    " macro highlight
    if vimentry#check('enable_macrohl', 'true')
        " TODO: silent call g:exMH_InitMacroList(g:exES_Macro)
    endif

    " TODO:
    " " set vimentry references
    " if !vimentry#check('sub_vimentry', '')
    "   for vimentry in g:exES_vimentryRefs
    "     let ref_entry_dir = fnamemodify( vimentry, ':p:h')
    "     let ref_vimfiles_dirname = '.vimfiles.' . fnamemodify( vimentry, ":t:r" )
    "     let fullpath_tagfile = exUtility#GetVimFile ( ref_entry_dir, ref_vimfiles_dirname, 'tag')
    "     if has ('win32')
    "       let fullpath_tagfile = exUtility#Pathfmt( fullpath_tagfile, 'windows' )
    "     elseif has ('unix')
    "       let fullpath_tagfile = exUtility#Pathfmt( fullpath_tagfile, 'unix' )
    "     endif
    "     if findfile ( fullpath_tagfile ) != ''
    "       let &tags .= ',' . fullpath_tagfile
    "     endif
    "   endfor
    " endif

    " finally we need to generate shell scripts for :Update command
    " TODO: call exUtility#CreateQuickGenProject ()

    " ===================================
    " run customized scripts
    " ===================================

    if exists('*g:exvim_post_init')
        call g:exvim_post_init()
    endif

    " ===================================
    " layout windows
    " ===================================

    " open project window
    if vimentry#check('enable_project_browser', 'true')
        let project_browser = vimentry#get( 'project_browser' )
        let g:ex_project_file = g:exvim_folder . "/files.exproject"

        " NOTE: Any windows open or close during VimEnter will not invoke WinEnter,WinLeave event
        " this is why I manually call doautocmd here
        if project_browser == 'ex'
            if exists ( ':NERDTreeClose' )
                exec 'NERDTreeClose'
            endif

            " open ex_project window
            doautocmd BufLeave
            doautocmd WinLeave

            silent call exproject#set_file_filters( vimentry#get('file_filter',[]) )
            silent call exproject#set_file_ignore_patterns( vimentry#get('file_ignore_pattern',[]) )
            silent call exproject#set_folder_filters( vimentry#get('folder_filter',[]) )
            silent call exproject#set_folder_filter_mode( vimentry#get('folder_filter_mode','include') )

            silent exec 'EXProject ' . fnameescape(g:ex_project_file)

            " TODO: add dirty message in ex-project window and hint user to press \R for refresh

            " bind key mapping
            if mapcheck('<leader>fc','n') != ""
                nunmap <leader>fc
            endif
            nnoremap <unique> <leader>fc :EXProjectFind<CR>

            if has('gui_running')
                if has ('mac')
                    if mapcheck('Ø','n') != ""
                        nunmap Ø
                    endif
                    nnoremap <unique> Ø :EXProjectOpen<CR>:redraw<CR>/
                else
                    if mapcheck('<M-O>','n') != ""
                        nunmap <M-O>
                    endif
                    nnoremap <unique> <M-O> :EXProjectOpen<CR>:redraw<CR>/
                endif
            endif

            " back to edit window
            doautocmd BufLeave
            doautocmd WinLeave
            call ex#window#goto_edit_window()

        elseif project_browser == 'nerdtree'
            if exists ( ':EXProjectClose' )
                exec 'EXProjectClose'
            endif

            " Example: let g:NERDTreeIgnore=['.git$[[dir]]', '.o$[[file]]']
            let g:NERDTreeIgnore = [] " clear ignore list
            let file_ignore_pattern = vimentry#get('file_ignore_pattern')
            if type(file_ignore_pattern) == type([])
                for pattern in file_ignore_pattern
                    silent call add ( g:NERDTreeIgnore, pattern.'[[file]]' )
                endfor
            endif

            if vimentry#check( 'folder_filter_mode',  'exclude' )
                let folder_filter = vimentry#get('folder_filter')
                if folder_filter == type([])
                    for pattern in folder_filter
                        silent call add ( g:NERDTreeIgnore, pattern.'[[dir]]' )
                    endfor
                endif
            endif

            " bind key mapping
            if mapcheck('<leader>fc','n') != ""
                nunmap <leader>fc
            endif
            nnoremap <unique> <leader>fc :NERDTreeFind<CR>

            if has('gui_running') "  the <alt> key is only available in gui mode.
                if has ('mac')
                    if mapcheck('Ø','n') != ""
                        nunmap Ø
                    endif
                    nnoremap <unique> Ø :NERDTreeFind<CR>:redraw<CR>/
                else
                    if mapcheck('<M-O>','n') != ""
                        nunmap <M-O>
                    endif
                    nnoremap <unique> <M-O> :NERDTreeFind<CR>:redraw<CR>/
                endif
            endif

            " open nerdtree window
            doautocmd BufLeave
            doautocmd WinLeave
            silent exec 'NERDTree'

            " back to edit window
            doautocmd BufLeave
            doautocmd WinLeave
            call ex#window#goto_edit_window()
        endif
    endif
endfunction

" exconfig#gen_sh_update_files {{{
function exconfig#gen_sh_update_files(path)
    " generate scripts
    if ex#os#is('windows')
        " check if gawk command executable
        if !executable('gawk')
            call ex#warning("Can't find gawk command in your system. Please install it first!")
        endif

        let folder_pattern = ""
        let use_folder_pattern = 0
        for name in g:exvim_root_folders
            let folder_pattern .= '"' . name . '",'
        endfor
        if !empty(g:exvim_root_folders)
            let use_folder_pattern = 1
            let folder_pattern = strpart( folder_pattern, 0, len(folder_pattern) - 1)
        endif

        let file_pattern = ''
        let file_filters = vimentry#get('file_filter', [])
        if !empty(file_filters)
            for name in file_filters
                let file_pattern .= '*.' . toupper(name) . ' '
            endfor
        else
            let file_pattern=''
        endif

        let file_pattern_gawk = ''
        let file_filters = vimentry#get('file_filter', [])
        if !empty(file_filters)
            for name in file_filters
                let file_pattern_gawk .= '\\.' . name . '$|'
            endfor
            let file_pattern_gawk = strpart( file_pattern_gawk, 0, len(file_pattern_gawk) - 1)
        else
            let file_pattern_gawk=''
        endif

        let fullpath = a:path . '/update-filelist.bat'
        let winpath = ex#path#translate(a:path,'windows')
        let wintoolpath = ex#path#translate(g:ex_tools_path,'windows')
        let wintoolpath = expand(wintoolpath)
        let scripts = [
                    \ '@echo off'                                       ,
                    \ 'set DEST='.winpath                               ,
                    \ 'set TOOLS='.wintoolpath                          ,
                    \ 'set FOLDERS='.folder_pattern                     ,
                    \ 'set USE_FOLDERS='.use_folder_pattern             ,
                    \ 'set FILE_SUFFIXS='.file_pattern                  ,
                    \ 'set FILE_FILTER_PATTERN="'.file_pattern_gawk.'"' ,
                    \ 'set TMP=%DEST%\_files_gawk'                      ,
                    \ 'set TMP2=%DEST%\_files'                          ,
                    \ 'set TARGET=%DEST%\files'                         ,
                    \ 'call %TOOLS%\shell\batch\update-filelist.bat'    ,
                    \ ]
    else
        let folder_pattern = ''
        for name in g:exvim_root_folders
            let folder_pattern .= './' . name . ','
        endfor
        if !empty(g:exvim_root_folders)
            let folder_pattern = strpart( folder_pattern, 0, len(folder_pattern) - 1)
        endif

        let file_pattern = ''
        let file_filters = vimentry#get('file_filter', [])
        if !empty(file_filters)
            for name in file_filters
                let file_pattern .= substitute(name, "\+", "\\\\+", "g") . '|'
            endfor
            let file_pattern = strpart( file_pattern, 0, len(file_pattern) - 1)
        else
            let file_pattern='.*'
        endif

        let fullpath = a:path . '/update-filelist.sh'
        let scripts = [
                    \ '#!/bin/bash'                                ,
                    \ 'export DEST="'.a:path.'"'                   ,
                    \ 'export TOOLS="'.expand(g:ex_tools_path).'"' ,
                    \ 'export FOLDERS="'.folder_pattern.'"'        ,
                    \ 'export FILE_SUFFIXS="'.file_pattern.'"'     ,
                    \ 'export TMP="${DEST}/_files"'                ,
                    \ 'export TARGET="${DEST}/files"'              ,
                    \ 'sh ${TOOLS}/shell/bash/update-filelist.sh'  ,
                    \ ]
    endif

    " save to file
    call writefile ( scripts, fullpath )
endfunction

" exconfig#gen_sh_update_ctags {{{
function exconfig#gen_sh_update_ctags(path)
    " get ctags cmd
    let ctags_cmd = 'ctags'
    if executable('ctags')
        let ctags_cmd = 'ctags'
    elseif executable('exuberant-ctags')
        " On Debian Linux, exuberant ctags is installed as exuberant-ctags
        let ctags_cmd = 'exuberant-ctags'
    elseif executable('exctags')
        " On Free-BSD, exuberant ctags is installed as exctags
        let ctags_cmd = 'exctags'
    elseif executable('ctags.exe')
        let ctags_cmd = 'ctags.exe'
    elseif executable('tags')
        let ctags_cmd = 'tags'
    else
        call ex#warning("Can't find ctags command in your system. Please install it first!")
    endif

    " get ctags options
    let ctags_optioins = '--fields=+iaS --extra=+q'
    " TODO:

    " generate scripts
    if ex#os#is('windows')
        let fullpath = a:path . '/update-tags.bat'
        let winpath = ex#path#translate(a:path,'windows')
        let wintoolpath = ex#path#translate(g:ex_tools_path,'windows')
        let wintoolpath = expand(wintoolpath)
        let scripts = [
                    \ '@echo off'                                ,
                    \ 'set DEST='.winpath                        ,
                    \ 'set TOOLS='.wintoolpath                   ,
                    \ 'set CTAGS_CMD='.ctags_cmd                 ,
                    \ 'set OPTIONS='.ctags_optioins              ,
                    \ 'set TMP=%DEST%\_tags'                     ,
                    \ 'set TARGET=%DEST%\tags'                   ,
                    \ 'call %TOOLS%\shell\batch\update-tags.bat' ,
                    \ ]
    else
        let fullpath = a:path . '/update-tags.sh'
        let scripts = [
                    \ '#!/bin/bash'                                ,
                    \ 'export DEST="'.a:path.'"'                   ,
                    \ 'export TOOLS="'.expand(g:ex_tools_path).'"' ,
                    \ 'export CTAGS_CMD="'.ctags_cmd.'"'           ,
                    \ 'export OPTIONS="'.ctags_optioins.'"'        ,
                    \ 'export TMP="${DEST}/_tags"'                 ,
                    \ 'export TARGET="${DEST}/tags"'               ,
                    \ 'sh ${TOOLS}/shell/bash/update-tags.sh'      ,
                    \ ]
    endif

    " save to file
    call writefile ( scripts, fullpath )
endfunction

" exconfig#gen_sh_update_symbols {{{
function exconfig#gen_sh_update_symbols(path)
    " check if gawk command executable
    if !executable('gawk')
        call ex#warning("Can't find gawk command in your system. Please install it first!")
    endif

    " generate scripts
    if ex#os#is('windows')
        let fullpath = a:path . '/update-symbols.bat'
        let winpath = ex#path#translate(a:path,'windows')
        let wintoolpath = ex#path#translate(g:ex_tools_path,'windows')
        let wintoolpath = expand(wintoolpath)
        let scripts = [
                    \ '@echo off'                                   ,
                    \ 'set DEST='.winpath                           ,
                    \ 'set TOOLS='.wintoolpath                      ,
                    \ 'set TMP=%DEST%\_symbols'                     ,
                    \ 'set TARGET=%DEST%\symbols'                   ,
                    \ 'call %TOOLS%\shell\batch\update-symbols.bat' ,
                    \ ]
    else
        let fullpath = a:path . '/update-symbols.sh'
        let scripts = [
                    \ '#!/bin/bash'                                ,
                    \ 'export DEST="'.a:path.'"'                   ,
                    \ 'export TOOLS="'.expand(g:ex_tools_path).'"' ,
                    \ 'export TMP="${DEST}/_symbols"'              ,
                    \ 'export TARGET="${DEST}/symbols"'            ,
                    \ 'sh ${TOOLS}/shell/bash/update-symbols.sh'   ,
                    \ ]
    endif

    " save to file
    call writefile ( scripts, fullpath )
endfunction

" exconfig#gen_sh_update_idutils {{{
function exconfig#gen_sh_update_idutils(path)
    " check if mkid command executable
    if !executable('mkid')
        call ex#warning("Can't find mkid command in your system. Please install it first!")
    endif

    " get exclude folder filter options
    " NOTE: this mkid have bug that if a folder name has white space, --prune="foo bar" will treat it as two folder.
    let exclude_folders = ''
    for name in g:exvim_root_exclude_folders
        let exclude_folders .= name . ' '
    endfor
    if !empty(g:exvim_root_folders)
        let exclude_folders = strpart( exclude_folders, 0, len(exclude_folders) - 1)
    endif

    " generate scripts
    if ex#os#is('windows')
        let fullpath = a:path . '/update-idutils.bat'
        let winpath = ex#path#translate(a:path,'windows')
        let wintoolpath = ex#path#translate(g:ex_tools_path,'windows')
        let wintoolpath = expand(wintoolpath)
        let scripts = [
                    \ '@echo off'                                   ,
                    \ 'set DEST='.winpath                           ,
                    \ 'set TOOLS='.wintoolpath                      ,
                    \ 'set EXCLUDE_FOLDERS='.exclude_folders        ,
                    \ 'set TMP=%DEST%\_ID'                          ,
                    \ 'set TARGET=%DEST%\ID'                        ,
                    \ 'call %TOOLS%\shell\batch\update-idutils.bat' ,
                    \ ]
    else
        let fullpath = a:path . '/update-idutils.sh'
        let scripts = [
                    \ '#!/bin/bash'                                  ,
                    \ 'export DEST="'.a:path.'"'                     ,
                    \ 'export TOOLS="'.expand(g:ex_tools_path).'"'   ,
                    \ 'export EXCLUDE_FOLDERS="'.exclude_folders.'"' ,
                    \ 'export TMP="${DEST}/_ID"'                     ,
                    \ 'export TARGET="${DEST}/ID"'                   ,
                    \ 'sh ${TOOLS}/shell/bash/update-idutils.sh'     ,
                    \ ]
    endif

    " save to file
    call writefile ( scripts, fullpath )

    " generate id-lang-autogen.map
    let file_filters = vimentry#get('file_filter',[])
    if !empty(file_filters) 
        let fullpath = a:path . '/id-lang-autogen.map'
        if ex#os#is('windows')
            let fullpath = ex#path#translate(fullpath,'windows')
        endif
        let scripts = [
                    \ '# autogen id-lang.map',
                    \ '*~                    IGNORE',
                    \ '*.bak                 IGNORE',
                    \ '*.bk[0-9]             IGNORE',
                    \ '[sp].*                IGNORE',
                    \ '*/.deps/*             IGNORE',
                    \ '*/.svn/*              IGNORE',
                    \ '*.svn-base            IGNORE',
                    \ '.git/*                IGNORE',
                    \ '.exvim*/*             IGNORE',
                    \ '*.err                 IGNORE',
                    \ '*.exe                 IGNORE',
                    \ '*.lnk                 IGNORE',
                    \ ]
        for item in file_filters 
            if item == ''
                continue
            endif
            silent call add ( scripts, '*.'.item.'    text')
        endfor

        " save to file
        call writefile( scripts, fullpath, 'b' )
    else
        let fullpath = a:path . '/id-lang-autogen.map'
        if findfile ( fullpath ) != ''
            call delete(fullpath)
        endif
    endif
endfunction

" exconfig#update_exvim_files {{{
function exconfig#update_exvim_files()
    if ex#os#is('windows')
        let shell_exec = 'call'
        let shell_and = ' & '
        let shell_pause = ' && pause'
        let suffix = '.bat'
        let path = '.\.exvim.'.g:exvim_project_name.'\'
    else
        let shell_exec = 'sh'
        let shell_and = ' && '
        let shell_pause = ''
        let suffix = '.sh'
        let path = './.exvim.'.g:exvim_project_name.'/'
    endif

    let cmd = ''
    let and = ''

    " update filelist, tags, symbols
    if vimentry#check('enable_tags','true')
        let cmd = shell_exec . ' ' . path.'update-filelist'.suffix
        let and = shell_and

        let cmd .= and
        let cmd .= shell_exec . ' ' . path.'update-tags'.suffix

        let cmd .= and
        let cmd .= shell_exec . ' ' . path.'update-symbols'.suffix
    endif

    " update IDs
    if vimentry#check('enable_gsearch','true')
        let cmd .= and
        let cmd .= shell_exec . ' ' . path.'update-idutils'.suffix
        let and = shell_and
    endif

    let cmd .= shell_pause
    call ex#hint('exVim Updating...')
    exec '!' . cmd
    call ex#hint('exVim Update Finish!')
endfunction



